#include "MemoryLeak.h"

#include <malloc.h>

// ----------------------------------------------------------------------------
// MemoryLea
// ----------------------------------------------------------------------------
MemoryLeak g_memoryLeak;

MemoryLeak::MemoryLeak()
{
  clear();
}

MemoryLeak::~MemoryLeak()
{
  _clear(0);
}

void MemoryLeak::_clear(int cnt)
{
  if (m_info != NULL)
  {
    ::free(m_info);
    m_info = NULL;
  }
  if (cnt != 0)
  {
    m_info = (MemoryLeakInfo*) ::malloc(sizeof(MemoryLeakInfo) * cnt);
    for (int i = 0; i < cnt; i++)
    {
      MemoryLeakInfo* info = &m_info[i];
      info->p    = NULL;
      info->size = 0;
      info->file = NULL;
      info->line = 0;
    }
  }
  m_cnt = cnt;
}

void MemoryLeak::_check()
{
  for (int i = 0; i < m_cnt; i++)
  {
    MemoryLeakInfo* info = &m_info[i];
    if (info->p != NULL)
    {
      printf("memory leak %p(%u) %s:%d\n", info->p, info->size, info->file, info->line);
    }
  }
}

MemoryLeakInfo* MemoryLeak::find(void* p)
{
  for (int i = 0; i < m_cnt; i++)
  {
    MemoryLeakInfo* info = &m_info[i];
    if (info->p == p)
    {
      return info;
    }
  }
  return NULL;
}

MemoryLeakInfo* MemoryLeak::add(void* p, size_t size, char* file, int line)
{
  for (int i = 0; i < m_cnt; i++)
  {
    MemoryLeakInfo* info = &m_info[i];
    if (info->p == NULL)
    {
      info->p    = p;
      info->size = size;
      info->file = file;
      info->line = line;
      return info;
    }
  }
  return NULL;
}

MemoryLeakInfo* MemoryLeak::del(void* p)
{
  for (int i = 0; i < m_cnt; i++)
  {
    MemoryLeakInfo* info = &m_info[i];
    if (info->p == p)
    {
      info->p    = NULL;
      info->size = 0;
      info->file = NULL;
      info->line = 0;
      return info;
    }
  }
  return NULL;
}

void MemoryLeak::clear(int cnt)
{
  g_memoryLeak._clear(cnt);
}

void MemoryLeak::check()
{
  g_memoryLeak._check();
}

class AutoInitializer
{
public:
  AutoInitializer()          { MemoryLeak::clear(); }
  virtual ~AutoInitializer() { MemoryLeak::check(); }
};

AutoInitializer autoInitializer; // gilgil temp 2011.09.03
