// ----------------------------------------------------------------------------
//
// VDream Component Suite version 7.0
//
// http://www.gilgil.net
//
// Copyright (c) Gilbert Lee All rights reserved
//
// ----------------------------------------------------------------------------

#ifndef __MEMORY_LEAK_H__
#define __MEMORY_LEAK_H__

#include <malloc.h>
#include <stdio.h> // for size_t

// ----------------------------------------------------------------------------
// MemoryLeakInfo
// ----------------------------------------------------------------------------
typedef struct _MemoryLeakInfo
{
  void*  p;
  size_t size;
  char*  file;
  int    line;
} MemoryLeakInfo;

// ----------------------------------------------------------------------------
// MemoryLeak
// ----------------------------------------------------------------------------
class MemoryLeak
{
  friend void* my_malloc(size_t size, const char* file, const int line);
  friend void my_free(void* p, const char* file, const int line);

protected:
  static const int MEMORY_LEAK_CNT = 65536;
  MemoryLeakInfo* m_info;
  int             m_cnt;

public:
  MemoryLeak();
  virtual ~MemoryLeak();

protected:
  void _clear(int cnt = MEMORY_LEAK_CNT);
  void _check();

protected:
  MemoryLeakInfo* find(void* p);
  MemoryLeakInfo* add (void* p, size_t size, char* file, int line);
  MemoryLeakInfo* del (void* p);

public:
  static void clear(int cnt = MEMORY_LEAK_CNT);
  static void check();
};

#endif // __MEMORY_LEAK_H__

